[![License](https://img.shields.io/badge/License-Apache%202.0-blue.svg)](https://opensource.org/licenses/Apache-2.0)
[![License: GPL v2](https://img.shields.io/badge/License-GPL%20v2-blue.svg)](https://www.gnu.org/licenses/old-licenses/gpl-2.0.en.html)
[![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT)
>
> Copyright 2021-2022 Xilinx, Inc
>
> Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License. You may obtain a copy of the License at
>
>     http://www.apache.org/licenses/LICENSE-2.0
>
> Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License.
>
> Specific files in this distribution are distributed with alternative licenses: GPLv2, MIT, and Public Domain. Please see the file LICENSES for details.

# Introduction 
This software package provides multiple separate services that can be incorporated into a user design, which can be a tandem design:

    1. Programming QSPI flash attached to a Versal family device from an x86 host over PCIe
    2. Multiboot using fallback to the next image in QSPI when a PL signal is asserted
    3. DDR controller tests

A companion PowerPoint provides the details of usage, please contact your Xilinx FAE for a copy.

# Programming QSPI
    - An x86 application 'x86_app' and multiple flavors of a companion agent application are provided for this purpose.
    - The user has to:
        - Decide whether to use R5_0 or A72_0 to implement the agent side of the code
        - Expose TCM and the OCM in one PCIe BAR for communication between the agent and the x86
        - Incorporate an agent app supporting programming in the second stage (tandem PROM) image. Tandem PCIe is currently not supported.
        - Update 'images.csv' and 'boards.csv' to teach the x86 app about your images and board parameters
        - Use 'x86_app' to program QSPI

# UIO example driver
    - Some host distributions may not allow access to the default 'resourceN' file mapped by x86_app.
    - An example UIO driver is provided that illustrates mapping the requisite bar in that case
    - It is modified from an existing UIO driver in the Linux kernel
    - The user has to modify the driver as needed for their use case
    - Usage of the UIO driver is not mandatory

# Multiboot
    Multiboot is implemented when EMIO indicates the need.
    The multiboot register is incremented by 1 to emulate fallback.

    The user has to:
        - Pre-program a fallback image in QSPI for the multiboot agent
        - Incorporate an agent app supporting multiboot in the second stage image

# Combined functionality
    An agent app can provide both programming and multiboot functionality.  The following agent applications are provided as examples:
        - 'prog_multiboot_app' provides programming and multiboot functionality
        - 'prog_app' provides programming functionality
        - 'multiboot_app' provides multiboot functionality

# DDR Controller test
    This test needs to run on the second R5/A72 agent processor

# Packaging
    Source code for all apps and associated run-time files are included in the source package.

# Bootstrapping an empty QSPI
    Program the new PDI (BIN) using JTAG, it should contain 'prog_app'
    Reboot the host without power cycling the chassis/board to reenumerate PCIe
    Run 'sudo lspci -vvv' to confirm the board is visible on PCIe
    Run x86_app with the requisite args to program qspi images

# Factory QSPI image for manufacturing
    The simulation mode of x86_app allows usage without a board. It creates a persistent binary file on the host that contains the QSPI image. Once it is programmed via simulation with all images, the file can be handed off for manufacturing.
     
# Building the source package
    You need Vitis 2021.2 and gcc installed on your host. You also need passwordless ssh login to a test machine containing a test board.
        - A convenience script is provided in the scripts directory that builds and packages everything: build.sh
        - The file 'userconfig.sh' in the same directory is for the user configrable parameters
